@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: (GPL-2.0-or-later OR BSD-2-Clause)
 /*
  * Traceshark - a visualizer for visualizing ftrace and perf traces
- * Copyright (C) 2017, 2019, 2020  Viktor Rosendahl <viktor.rosendahl@gmail.com>
+ * Copyright (C) 2017, 2019-2021  Viktor Rosendahl <viktor.rosendahl@gmail.com>
  *
  * This file is dual licensed: you can use it either under the terms of
  * the GPL, or the BSD license, at your option.
@@ -65,7 +65,7 @@ public:
 		FILTER_EVENT,
 		FILTER_TIME,
 		FILTER_CPU,
-		FILTER_ARG,
+		FILTER_REGEX,
 		NR_FILTERS
 	} filter_t;
 	void enable(filter_t filter);
