@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: (GPL-2.0-or-later OR BSD-2-Clause)
 /*
  * Traceshark - a visualizer for visualizing ftrace and perf traces
- * Copyright (C) 2015-2020  Viktor Rosendahl <viktor.rosendahl@gmail.com>
+ * Copyright (C) 2015-2021  Viktor Rosendahl <viktor.rosendahl@gmail.com>
  *
  * This file is dual licensed: you can use it either under the terms of
  * the GPL, or the BSD license, at your option.
@@ -55,7 +55,7 @@
 
 #include "gitversion.h"
 
-#define TRACESHARK_VERSION_STRING "0.9.11-beta" TRACESHARK_GIT_VERSION
+#define TRACESHARK_VERSION_STRING "0.9.12-beta" TRACESHARK_GIT_VERSION
 
 #include <QtCore>
 #include <cstdint>
@@ -79,6 +79,12 @@
 #define DEFAULT_MAX_VRT_LATENCY (20)
 #define MIN_MAX_VRT_LATENCY (1)
 #define MAX_MAX_VRT_LATENCY (1000)
+#define DEFAULT_MAINWINDOW_WIDTH (1800)
+#define MAX_MAINWINDOW_WIDTH (15360)
+#define MIN_MAINWINDOW_WIDTH (320)
+#define DEFAULT_MAINWINDOW_HEIGHT (950)
+#define MAX_MAINWINDOW_HEIGHT (8640)
+#define MIN_MAINWINDOW_HEIGHT (200)
 
 #ifdef QCUSTOMPLOT_USE_OPENGL
 #define has_opengl() (true)
@@ -146,8 +152,27 @@ typedef enum : int {
 #define SPROL32(VALUE, N) \
 	((VALUE << N) | (VALUE >> (32 - N)))
 
+#define TSHARK_LOGIC_DEFS_			\
+		TSHARK_LOGIC_ITEM_(OR),		\
+		TSHARK_LOGIC_ITEM_(NOR),	\
+		TSHARK_LOGIC_ITEM_(XOR),	\
+		TSHARK_LOGIC_ITEM_(XNOR),	\
+		TSHARK_LOGIC_ITEM_(AND),	\
+		TSHARK_LOGIC_ITEM_(NAND)
+
+
 namespace TShark {
 
+#undef TSHARK_LOGIC_ITEM_
+#define TSHARK_LOGIC_ITEM_(a) LOGIC_##a
+	enum Logic : int {
+		TSHARK_LOGIC_DEFS_,
+		LOGIC_NR
+	};
+#undef TSHARK_LOGIC_ITEM_
+
+	extern const char * const logic_names[];
+
 	enum CursorIdx {RED_CURSOR, BLUE_CURSOR, NR_CURSORS};
 
 	union value32 {
@@ -175,6 +200,8 @@ namespace TShark {
 					    const struct timespec &s2) {
 		return s1.tv_sec == s2.tv_sec && s1.tv_nsec == s2.tv_nsec;
 	}
+
+	extern QString translateRegexError(int ecode);
 }
 
 #endif /* TRACESHARK_H */
